
#include "shm_memory_tool.h"

static cList<PointerShmFdListObject> SHMMemoryMappings;

static unsigned int shm_number = 0;

void * shm_malloc (size_t size)
{
    int    shmfd=-1;
    void * ptr=NULL;
    char name[255];

    snprintf( name, 255, "/vdr_memory_%8.8u", shm_number++);

    if ((shmfd = shm_open(name, O_RDWR|O_CREAT, S_IRUSR|S_IWUSR)) < 0) {
	esyslog("shm_malloc shm_open of %s failed: %s\n", name, strerror(errno));
	goto err;
    }

    if(ftruncate(shmfd, size) < 0) {
	esyslog("shm_open ftruncate of %s to size %d failed: %s\n",
		 name, size, strerror(errno));
	goto err;
    }

    ptr = mmap(NULL, size, 
	       PROT_READ|PROT_WRITE, 
	       MAP_PRIVATE|MAP_NORESERVE|MAP_LOCKED,
	       shmfd, 0);

    if( ptr == MAP_FAILED) {
	esyslog("shm_open mmap of %s to size %d failed: %s\n",
		 name, size, strerror(errno));
	goto err;
    }

    SHMMemoryMappings.Add(new PointerShmFdListObject( shmfd, name, ptr, size));
    SHMMemoryMappings.Sort();

    return ptr;

err:
    if (ptr) {
	(void) munlock(ptr, size);
	(void) munmap (ptr, size);
    }
    ptr=NULL;
    if (shmfd >= 0) {
	shm_unlink(name);
	close(shmfd);
    }
    shmfd=-1;
    return NULL;
}

void   shm_free   (void * memptr)
{
    PointerShmFdListObject * p_item = shm_find (memptr );

    if ( p_item == NULL ) {
	esyslog("shm_free pointer %p not found in alloc space\n", memptr);
    	return;
    }

    if (p_item->getPtr()) {
	(void) munlock(p_item->getPtr(), p_item->getSize());
	(void) munmap (p_item->getPtr(), p_item->getSize());
    }
    if (p_item->getFd() >= 0) {
	shm_unlink(p_item->getFileName());
	close(p_item->getFd());
    }

    SHMMemoryMappings.Del(p_item);
}

PointerShmFdListObject  * shm_find   (void * memptr )
{
    PointerShmFdListObject * p_item = NULL;
    int i;

    for ( i=SHMMemoryMappings.Count()-1; i >= 0; i-- ) {
    	p_item = (PointerShmFdListObject *)SHMMemoryMappings.Get(i);
	if ( p_item->getPtr() == memptr )
		break;
    }

    return p_item;
}

